#!/bin/sh
#
# A license is hereby granted to reproduce this software source code
# and to create executable versions from this source code for
# personal, non-commercial use. The copyright notice included with the
# software must be maintained in all copies produced.
#
# THIS PROGRAM IS PROVIDED "AS IS". THE AUTHOR PROVIDES NO WARRANTIES
# WHATSOEVER, EXPRESSED OR IMPLIED, INCLUDING WARRANTIES OF
# MERCHANTABILITY, TITLE, OR FITNESS FOR ANY PARTICULAR PURPOSE. THE
# AUTHOR DOES NOT WARRANT THAT USE OF THIS PROGRAM DOES NOT INFRINGE
# THE INTELLECTUAL PROPERTY RIGHTS OF ANY THIRD PARTY IN ANY COUNTRY.
#
# So there.
#
# Copyright (c) 1994-2006, John Conover, All Rights Reserved.
#
# Partial key search a tab delimited flat file database of phone
# numbers, and if a unique match is found, dial the number using the
# spcid(1) program.
#
# The default database of phone numbers is "${HOME}/.phone.numbers".
#
# The database records consist of exactly three fields:
#
#     1) The name, (first name followed by last name, separated by
#        spaces.
#
#     2) The phone, (cellular, pager, home, work, etc.)
#
#     3) The phone number.
#
# The fields are separated by a single tab character, (hex 09).
#
# The partial key search begins with each field, and all records that
# match all partial keys are printed on stderr, unless only one record
# matches, in which case the number is dialed.
#
############################################################
#
# $Revision: 0.0 $
# $Date: 2006/06/18 08:33:13 $
# $Id: phone.sh,v 1.0 2006/06/18 08:33:13 conover Exp $
# $Log: phone.sh,v $
# Revision 0.0  2006/06/18 08:33:13  conover
# Initial revision
#
# Tab character, hex 09:
#
TAB='[	]'
#
CALL="1" # dial the number, 1 = yes, 0 = now
RETVAL="0" # return value, assume no error
#
DATABASE="${HOME}/.phone.numbers"
#
while [ "$#" -gt "0" ]
do
    case "${1}" in
        -d) # request for database file name?
            shift
            if [ "$#" -gt "0" ]
            then
                DATABASE="$1"
                if [ ! -e "${DATABASE}" ]
                then
                    echo "Database file does not exist" 1>&2
                    RETVAL="1"
                fi
                shift
            else
                echo "No database file name specified" 1>&2
                RETVAL="1"
            fi
            ;;
        -q) # request for displaying query without dialing?
            shift
            CALL="0"
            ;;
         *) # anything else is an error
            break
            ;;
    esac
done
#
if [ "${RETVAL}" = "0" ] # any errors?
then
    if NUMBER=`echo "${*}" | egrep '^ *(\+? *1?[-.( ]*[0-9][0-9][0-9][-.) ]*)?[0-9][0-9][0-9][-. ]*[0-9][0-9][0-9][0-9] *$'`
    then
        # telephone number
        RETVAL="0"
    else
        # wasn't a telephone number, check for a name in the database
        case "${#}" in
            "1") # single partial key:
                 #   first name
                 #   last name
                 #
                 NUMBER=`egrep -i "^((${1})|([-a-z. ]+[- ]${1}))" "${DATABASE}"`
                 ;;
            "2") # two partial keys:
                 #   first name, last name
                 #   last name, first name
                 #   first name, phone
                 #   last name, phone
                 #
                 NUMBER=`egrep -i "^((${1}[-a-z. ]*[- ]${2})|(${2}[-a-z. ]*[- ]${1})|(${1}[-a-z. ]*${TAB}${2})|([-a-z. ]+[- ]${1}[-a-z. ]*${TAB}${2}))" "${DATABASE}"`
                 ;;
            "3") # three partial keys:
                 #   first name, last name, phone
                 #   last name, first name, phone
                 #
                 NUMBER=`egrep -i "^((${1}[-a-z. ]*[- ]${2}[-a-z. ]*${TAB}${3})|(${2}[-a-z. ]*[- ]${1}[-a-z. ]*${TAB}${3}))" "${DATABASE}"`
                 ;;
              *) # none of the above:
                 RETVAL="1"
                 ;;
        esac
    fi
    #
    if [ "${RETVAL}" = "0" ] # any errors?
    then
        if [ "${NUMBER}" != "" ] # number not null?
        then
            COUNT=`echo "$NUMBER" | wc -l` # how many records?
            #
            if [ "${COUNT}" = "1" ] # one record means valid lookup or phone number
            then
                if [ "${CALL}" = "1" ] # dial the number?
                then
                    echo "Calling: ${NUMBER}"
                    #
                    # Custom command to dial a number:
                    #
                    echo "${NUMBER}" | sed "s/^.*${TAB}.*${TAB}//"
                else # else print the number
                    echo "${NUMBER}"
                fi
                RETVAL="0" # success
            else
                echo "${NUMBER}" 1>&2
                RETVAL="1"
            fi
        else
            echo "No matching phone number" 1>&2
            RETVAL="1"
        fi
    else
        echo "Dial a phone number with partial key name lookup:" 1>&2
        echo "${0} [-q] phone number" 1>&2
        echo "${0} [-q] [-d database] first name" 1>&2
        echo "${0} [-q] [-d database] last name" 1>&2
        echo "${0} [-q] [-d database] first name, last name" 1>&2
        echo "${0} [-q] [-d database] lastname name, first name" 1>&2
        echo "${0} [-q] [-d database] first name, phone" 1>&2
        echo "${0} [-q] [-d database] last name, phone" 1>&2
        echo "${0} [-q] [-d database] first name, last name, phone" 1>&2
        echo "${0} [-q] [-d database] last name, first name, phone" 1>&2
    fi
fi
#
exit "${RETVAL}"
