#!/usr/bin/perl -wT
#
# ------------------------------------------------------------------------------
#
# A license is hereby granted to reproduce this software source code and
# to create executable versions from this source code for personal,
# non-commercial use.  The copyright notice included with the software
# must be maintained in all copies produced.
#
# THIS PROGRAM IS PROVIDED "AS IS". THE AUTHOR PROVIDES NO WARRANTIES
# WHATSOEVER, EXPRESSED OR IMPLIED, INCLUDING WARRANTIES OF
# MERCHANTABILITY, TITLE, OR FITNESS FOR ANY PARTICULAR PURPOSE.  THE
# AUTHOR DOES NOT WARRANT THAT USE OF THIS PROGRAM DOES NOT INFRINGE THE
# INTELLECTUAL PROPERTY RIGHTS OF ANY THIRD PARTY IN ANY COUNTRY.
#
# Copyright (c) 1994-2006, John Conover, All Rights Reserved.
#
# Comments and/or bug reports should be addressed to:
#
#     john@email.johncon.com (John Conover)
#
#     http://www.johncon.com/ntropix/
#     http://www.johncon.com/
#
# ------------------------------------------------------------------------------
#
# A simple log file formatter for the CID log files produced by the
# spcid(1) program. The spcid log file is /var/log/cid/spcid.log.
#
# $Revision: 0.0 $
# $Date: 2006/06/18 19:36:00 $
# $Id: spcid.pl,v 1.6 2006/06/18 19:36:00 john Exp $
# $Log: spcid.pl,v $
# Revision 0.0  2006/06/18 19:36:00  john
# Initial revision
#
#
use strict;
#
$= = 32767; # no page eject = ^L
#
# The variables:
#
my $date = ''; # the CID date string
my $time = ''; # the CID time string
my $name = ''; # the CID name string, name, or O for unknown, or P for private
my $number = ''; # the CID phone number string, O for out of area
my $u_time = ''; # Epoc time
my $l_time = ''; # local time, converted from Epoc time
#
# Format of top two lines of report:
#
format HEADER =
Name               Number            Date / Time
-------------------------------------------------------------
.
#
# Format of a CID record:
#
format RECORD =
@<<<<<<<<<<<<<<<<< @<<<<<<<<<<<<<<   @<<<<<<<<<<<<<<<<<<<<<<<
$name,              $number,          $l_time
.
#
$^ = 'HEADER';
$~ = 'RECORD';
#
open (LOGFILE, "</var/log/cid/spcid.log"); # open the CID log file
#
while (<LOGFILE>) # while records from the log file:
{
    ($date, $time, $name, $number, $u_time) = split (/\t/); # parse the tab delimited fields in the record into the variables
    $l_time = localtime ($u_time); # convert Epoc time to local time
    $name =~ s/([A-Z])([A-Z]*)/$1\L$2/g; # de-capitalize all characters, except first character of words
    $name =~ s/^O$/Unknown Name/; # substitute unknown for O
    $name =~ s/^P$/Private Name/; # substitute private for P
    $number =~ s/^O$/Unknown Number/g; # substitute out of area for O
    $number =~ s/^P$/Private Number/g; # substitute out of area for O
    $number =~ s/(\d{3})(\d{3})(\d{4})/\($1\) $2-$3/; # make a user friendly version of phone number
    write; # print the record using RECORD format
}
